/*
 * Copyright (C) 2007 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.huawei.android.backup.backupRemoteService;

import com.huawei.android.backup.backupRemoteService.IRemoteServiceCallback;

/**
 * Backup AIDL remote service interface.
 * Version = 1
 * Supported MIN version = 1
 */
interface IRemoteService {
    /**
     * register client callback.
     * service will send client message by parameter IRemoteServiceCallback. 
     * @param clinetID caller ID, every binder use different id.
     * @return 0, success. If service was occupied by some other application's client, or service version
     * does not match client, will return error.
     * see {@link com.huawei.android.backup.backupRemoteService.BackupAIDLConstant.AIDLAPIResult}
     */
    int registerCallback(int clientID, int aidlVersion, IRemoteServiceCallback cb);
    
    /**
     * Remove a previously registered callback interface.
     * @return 0, success. If cb equals null or clinetID do not registered, return err.
     * see {@link com.huawei.android.backup.backupRemoteService.BackupAIDLConstant.AIDLAPIResult}
     */
    int unregisterCallback(int clinetID, IRemoteServiceCallback cb);

    /**
     * abort all processing missions
     * @return 0, success.
     */
    int abortDoing(int clientID);
    
    /**
     * Do backup.
     * @param clinetID caller ID, every binder use different id.
     * @param location Backup Super DIR, for example, /sdcard/HuaweiBackup/backupfiles.
     * @param fileName Backup data DIR, for example, 2015-01-01. The backup database will all store in this DIR.
     * @param modules Backup modules' logic name list. APP logic name is its package name. For example, contact, sms, 
     *                 picture, com.android.xxx.
     * @param parameter Backup process expend parameter, include encrypt information, action flag, contact account info, 
     *                backup APP mode and so on.
     * @return 0, success. 
     * see {@link com.huawei.android.backup.backupRemoteService.BackupAIDLConstant.AIDLAPIResult}
     */
    int doBackup(int clientID, String location, String fileName, in String[] modules, in Bundle parameter);
    
    /**
     * Do restore.
     * @param clinetID caller ID, every binder use different id.
     * @param location Backup file Super DIR, for example, /sdcard/HuaweiBackup/backupfiles.
     * @param fileName Backup file DIR, for example, 2015-01-01. The backup database will all store in this DIR.
     * @param modules Restore modules' logic name list. APP logic name is its package name. For example, contact, sms, 
     *                 picture, com.android.xxx.
     * @param parameter Restore process expend parameter, include encrypt information, action flag, contact account info, 
     *                media file restore location and so on.
     * @return 0, success. 
     * see {@link com.huawei.android.backup.backupRemoteService.BackupAIDLConstant.AIDLAPIResult}
     */
    int doRestore(int clientID, String location, String fileName, in String[] modules, in Bundle parameter);
    
    /**
     * Do restore one module.
     * @param clinetID caller ID, every binder use different id.
     * @param location Backup file Super DIR, for example, /sdcard/HuaweiBackup/backupfiles.
     * @param fileName Backup file DIR, for example, 2015-01-01. The backup database will all store in this DIR.
     * @param module Restore modules' logic name. APP logic name is its package name. For example, contact, sms, 
     *                 picture, com.android.xxx.
     * @param parameter Restore process expend parameter, include encrypt information, action flag, contact account info, 
     *                media file restore location and so on.
     * @return 0, success. 
     * see {@link com.huawei.android.backup.backupRemoteService.BackupAIDLConstant.AIDLAPIResult}
     */
    int doRestoreOneModule(int clientID, String location, String filename, String module, in Bundle parameter);
    
    /**
     * Get what can backup and count every item.
     * @param clinetID caller ID, every binder use different id.
     * @param module Restore modules' logic name. APP logic name is its package name. For example, contact, sms, 
     *                 picture, com.android.xxx.
     * @param parameter execute parameter, include action flag.
     * @return 0, success. 
     * see {@link com.huawei.android.backup.backupRemoteService.BackupAIDLConstant.AIDLAPIResult}
     */
    int getBackupModuleInfo(int clientID, in String[] modules, in Bundle parameter);
    
    /**
     * Get APP data and APK file size. It used for counting the backup file size before backup operation.
     * @param modules, the package name which want to get its size.
     * @return 0, success. See {@link com.huawei.android.backup.backupRemoteService.BackupAIDLConstant.AIDLAPIResult}
     */
    int getAppsSize(int clientID, in String[] modules);
    
    /**
     * Scan internal memory and external SDCard, get all CMCC backup Data DIRs
     * @param customPath the custom backup data DIR path on internal memory and SDcard,
     * it should not be add to result.
     * @return 0, success. See {@link com.huawei.android.backup.backupRemoteService.BackupAIDLConstant.AIDLAPIResult}
     */
    int getAllCMCCBackupFiles(int clientID, in String[] customPath);
    
    /**
     * Get all backup files list
     * @param path the backup file parent DIR path.
     * @return 0, success. See {@link com.huawei.android.backup.backupRemoteService.BackupAIDLConstant.AIDLAPIResult}
     */
    int getAllBackupFileList(int clientID, in String[] path, in Bundle parameter);
    
    /**
     * Get backup file summary, include encrypt info, backup device info, modules counting info.
     * @param paths the backup file DIR path.
     * @return 0, success. See {@link com.huawei.android.backup.backupRemoteService.BackupAIDLConstant.AIDLAPIResult}
     */
    int getBackupFileSummary(int clientID, in String[] paths, in Bundle parameter);
    
    /**
     * Get restore data modules details
     * @param location backup DIR parent DIR
     * @param fileName backup DIR's name
     * @param parameter execute parameter, include action flag.
     * @return 0, success. See {@link com.huawei.android.backup.backupRemoteService.BackupAIDLConstant.AIDLAPIResult}
     */
    int getRestoreModulesInfo(int clientID, String location, String fileName, in Bundle parameter);

    /**
     * Get restore APP size and check the risk to install.
     * @param location backup DIR parent DIR
     * @param fileName backup DIR's name
     * @param modules APP package name list
     * @param parameter execute parameter, include action flag.
     * @return 0, success. See {@link com.huawei.android.backup.backupRemoteService.BackupAIDLConstant.AIDLAPIResult}
     */
    int getRestoreAppInfo(int clientID, String location, String fileName, in String[] modules, in Bundle parameter);
    
    /**
     * Get new device environment information, include storage information, install APP information,
     * support modules and interface information and so on.
     * It only used by clone process.
     * @param parameter execute parameter, include action flag.
     * @return 0, success. See {@link com.huawei.android.backup.backupRemoteService.BackupAIDLConstant.AIDLAPIResult}
     */
    int getNewDeviceInfo(int clientID, in Bundle parameter);
    
    /**
     * Delete backup files. 
     * @param clinetID caller ID, every binder use different id.
     * @param location backup DIR parent DIR
     * @param fileName backup DIR's name
     * @param modules APP package name list
     * @param parameter execute parameter, include action flag.
     * @return 0, success. See {@link com.huawei.android.backup.backupRemoteService.BackupAIDLConstant.AIDLAPIResult}
     */
    int deleteBackupFiles(int clientID, in String[] location, in String[] fileName, in boolean[] isEncrypted);
    
    /**
     * Check APP risk info.
     * Some APP may be not supported to android 5, calling this API to call android.packageinstaller API
     * get APP risk info.
     * @param clinetID caller ID, every binder use different id.
     * @param appMap key is APP package name, value is APP version code.
     * @param parameter execute parameter, include action flag.
     * @return 0, success. See {@link com.huawei.android.backup.backupRemoteService.BackupAIDLConstant.AIDLAPIResult}
     */
    int checkAppRiskInfo(int clientID, in Bundle parameter);

    /**
     * Do authenticate before obtaining information or uploading to DBANK.
     * @param clinetID caller ID, every binder use different id.
     * @param token See HwAccount.
     * @param siteId See HwAccount.
     * @param userId See HwAccount.
     * @return 0, success. See {@link com.huawei.android.backup.backupRemoteService.BackupAIDLConstant.AIDLAPIResult}
     */
    int doUserAuthWithUserId(int clientID, String token, String siteId, String userId);
    
    /**
     * Do install FBService APP.
     * Call this method, Backup Service will install FBService APP background.
     * @param clinetID caller ID, every binder use different id.
     * @return 0, success. See {@link com.huawei.android.backup.backupRemoteService.BackupAIDLConstant.AIDLAPIResult}
     */
    int doInstallFBService(int clientID);
    
    /**
     * Check FBService state.
     * If this device is HuaweiDevice and FBService uninstalled by user, calling this
     * method will send an error message to caller. When caller received the message,
     * it should show reinstall dialog. If user make sure to reinstall FBService, calling
     * doInstallFBService().
     * @param clinetID caller ID, every binder use different id.
     * @return 0, success. See {@link com.huawei.android.backup.backupRemoteService.BackupAIDLConstant.AIDLAPIResult}
     */
    int checkFBSState(int clientID);
    
    /**
     * Check permissions.
     * On Android 6.0 version, Backup APP need request Runtime permissions. After Service booting,
     * calling this method to check that, whether to show request Runtime UI.
     * @return Permissions were not granted string list. If all permissions were granted, return 
     * empty String list.
     */
    String[] checkBackupRunTimePermissons();
    
    // add for cmcc branch xwx402519.
    boolean getBackupModuleNumber(String location, boolean onlyItem, int clientId);
    boolean getBackupItemNumberNew(String location, int clientId);
    boolean setParameter(in Map parameter, int clientId);
    boolean checkStorageState(boolean backupOrRestroe, String location, String fileName, in String[] modules, int clientId);
    /*z00226188 add for getAllPath*/
    String[] getLocation(int clientId);
    /*end*/
    boolean getRestoreModulesInfoFast(String location, String fileName, int clientId);
    boolean doUpdateBackupFiles(String location, int clientId);
    boolean getRestoreOldApkFilesSize(String Path, int clientId);
    boolean getRestoreFilesSizeNew(String location, int clientId);
    boolean doRestoreDecrypt(String location, String fileName, in String[] modules, boolean isDecrypt, String decryptPasswd, int clientId, in Bundle exeBundle);
    boolean doBackupEncrypt(String location, String fileName, in String[] modules, boolean isEncrypt, String encryptPasswd, String encryptMsg, int clientId);
    boolean doBackupNew(String location, String fileName, in String[] modules, int clientId);
    boolean getRestoreModulesAndTotalNew(String location, String fileName, int clientId);
    boolean getRestoreApksInfo(String location, String fileName, in String[] modules, int clientId);
    // add for cmcc branch xwx402519.
    
    // add for cmcc  zwx423888.
    int abortDoingCMCC(int clientID);
    /*备份服务可能会有多个模块使用，但是如果有一个模块已经使用clientID绑定了服务,
    则其他模块不能绑定备份的服务,因此提供该接口，用来释放自己的绑定（不需要使用服务时释放，
    让其他模块可以使用，当前的使用场景是：备份模块的自动备份和手动备份功能）*/
    void releaseCurrentBindingAppID();
    //共享文件夹路径
    void setShareFolderCurMountPath(String mShareFolderMountPath);
    //前台判断服务是否在停止中
    boolean isControlThreadAborting();
    /**
     * 获取因备份过程中发生异常导致生成的临时文件(垃圾文件，需要清理).
     */
    int getTempBackupFiles(int clientID, in String[] path, in Bundle parameter);
}
